@@ -16,7 +16,7 @@ class RRTStarPlanner(Planner):
     Args:
         robot(SingleArm or Bimanual): The manipulator robot type is SingleArm or Bimanual
         self_collision_manager: CollisionManager for robot's self collision check
-        obstacle_collision_manager: CollisionManager for collision check between robot and object
+        object_collision_manager: CollisionManager for collision check between robot and object
         delta_distance(float): distance between nearest vertex and new vertex
         epsilon(float): 1-epsilon is probability of random sampling
         gamma_RRT_star(int): factor used for search radius
@@ -28,7 +28,7 @@ class RRTStarPlanner(Planner):
         self, 
         robot,
         self_collision_manager=None,
-        obstacle_collision_manager=None,
+        object_collision_manager=None,
         delta_distance=0.5,
         epsilon=0.2,
         max_iter=3000,
@@ -37,8 +37,9 @@ class RRTStarPlanner(Planner):
         n_step=10
     ):
         super(RRTStarPlanner, self).__init__(
-            robot, self_collision_manager, 
-            obstacle_collision_manager,
+            robot, 
+            self_collision_manager, 
+            object_collision_manager,
             dimension
         )
         self.delta_dis = delta_distance
@@ -62,9 +63,18 @@ class RRTStarPlanner(Planner):
     def __repr__(self):
         return 'pykin.planners.rrt_star_planner.{}()'.format(type(self).__name__)
     
-
     @logging_time
-    def get_path_in_joinst_space(self, cur_q, goal_pose, max_iter=None, resolution=1):
+    def get_path_in_joinst_space(
+        self, 
+        cur_q,
+        goal_pose, 
+        max_iter=None, 
+        resolution=1, 
+        is_attached=False, 
+        obj_info=None,
+        T_between_gripper_and_obj=None,
+        object_collision_manager=None
+    ):
         """
         Get path in joint space
 
@@ -77,6 +87,14 @@ class RRTStarPlanner(Planner):
         if max_iter is not None:
             self._max_iter = max_iter
 
+        if is_attached:
+            self.object_c_manager = object_collision_manager
+            self.obj_info = obj_info
+            self.T_between_gripper_and_obj = T_between_gripper_and_obj
+            self.self_c_manager.add_object(
+                self.obj_info["name"], 
+                gtype="mesh", gparam=self.obj_info["gtype"], transform=self.obj_info["transform"])
+
         cnt = 0
         total_cnt = 10
 
@@ -103,13 +121,13 @@ class RRTStarPlanner(Planner):
                     logger.info(f"iter : {step}")
                     
                 rand_q = self.random_state()
-                if not self._collision_free(rand_q):
+                if not self._collision_free(rand_q, is_attached):
                     continue
 
                 nearest_q, nearest_idx = self.nearest_neighbor(rand_q, self.T)
                 new_q = self.new_state(nearest_q, rand_q)
     
-                if self._collision_free(new_q) and self._check_q_in_limits(new_q):
+                if self._collision_free(new_q, is_attached) and self._check_q_in_limits(new_q):
                     neighbor_indexes = self.get_near_neighbor_indices(new_q)
                     min_cost = self.get_new_cost(nearest_idx, nearest_q, new_q)
                     min_cost, nearest_idx = self.get_minimum_cost_and_index(neighbor_indexes, new_q, min_cost, nearest_idx)
@@ -197,20 +215,7 @@ class RRTStarPlanner(Planner):
         Returns:
             Norm(float or ndarray)
         """
-
-        theta1 = 0
-        theta2 = 0
-        dx, dy, dist = 0, 0, 0
-
-        for i in range(self.dimension):
-            theta1 += pointA[i]
-            theta2 += pointB[i]
-            dx += np.cos(theta1) - np.cos(theta2)
-            dy += np.sin(theta1) - np.sin(theta2)
-            dist += np.sqrt(dx * dx + dy * dy)
-
-        return dist * 1 / self.dimension
-        # return np.linalg.norm(pointB - pointA)
+        return np.linalg.norm(pointB - pointA)
 
     def new_state(self, nearest_q, random_q):
         """
@@ -322,7 +327,7 @@ class RRTStarPlanner(Planner):
             bool
         """
         dist = self.distance(point, self.goal_q)
-        if dist <= 0.5:
+        if dist <= 0.6:
             return True
         return False
 
